@@ -306,17 +306,23 @@ cal_backend_wcap_handle_import (CalBackendWCAP *wcap, ECalComponent *comp, CalOb
 				if (!start_equal (comp, existing, FALSE) ||
 						!end_equal (comp, existing, FALSE) ||
 						!rrules_equal (comp, existing)) {
+					char *rid;
+
+					rid = cal_backend_wcap_get_rid_string (existing);
 					/* Delete the old ones and let the server re-expand */
 					if (import_kind == ICAL_VEVENT_COMPONENT)
-						sunone_connection_deleteevents_by_id (cal_backend_wcap_get_connection (wcap),
-																cal_backend_wcap_get_calid (wcap),
-																uid, cal_backend_wcap_get_rid_string (existing),
-																cal_backend_wcap_to_sunone_mod (mod, existing));
+						sunone_connection_deleteevents_by_id (
+								cal_backend_wcap_get_connection (wcap),
+								cal_backend_wcap_get_calid (wcap),
+								uid, rid,
+								cal_backend_wcap_to_sunone_mod (mod, existing));
 					else if (import_kind == ICAL_VTODO_COMPONENT)
-						sunone_connection_deletetodos_by_id (cal_backend_wcap_get_connection (wcap),
-																cal_backend_wcap_get_calid (wcap),
-																uid, cal_backend_wcap_get_rid_string (existing),
-																cal_backend_wcap_to_sunone_mod (mod, existing));
+						sunone_connection_deletetodos_by_id (
+								cal_backend_wcap_get_connection (wcap),
+								cal_backend_wcap_get_calid (wcap),
+								uid, rid,
+								cal_backend_wcap_to_sunone_mod (mod, existing));
+					g_free (rid);
 
 					clone = icalcomponent_new_clone (import_comp);
 					prop = icalcomponent_get_first_property (clone, ICAL_RECURRENCEID_PROPERTY);
@@ -585,7 +591,7 @@ cal_backend_wcap_guess_method (CalBackendWCAP *wcap, EDataCal *cal, ECalComponen
 	return REPLY;
 }
 
-const char *
+char *
 cal_backend_wcap_get_rid_string (ECalComponent *comp)
 {
 	ECalComponentRange range;
@@ -593,18 +599,30 @@ cal_backend_wcap_get_rid_string (ECalComponent *comp)
 	
 	e_cal_component_get_recurid (comp, &range);
 	if (!range.datetime.value)
-		return "0";
+		return g_strdup ("0");
 	tt = *range.datetime.value;
 	e_cal_component_free_range (&range);
 
-	return icaltime_is_valid_time (tt) && !icaltime_is_null_time (tt) ? 
-		(const char *)icaltime_as_ical_string (tt) : "0";
+	if (icaltime_is_valid_time (tt) && !icaltime_is_null_time (tt))
+		return icaltime_as_ical_string (tt);
+	else
+		return g_strdup ("0");
 }
 
 gint
 cal_backend_wcap_instance_list_compare (gconstpointer a, gconstpointer b)
 {
-	return strcmp (cal_backend_wcap_get_rid_string (E_CAL_COMPONENT (a)), cal_backend_wcap_get_rid_string (E_CAL_COMPONENT (b)));
+	gint ret;
+	char *astr;
+	char *bstr;
+
+	astr = cal_backend_wcap_get_rid_string (E_CAL_COMPONENT (a));
+	bstr = cal_backend_wcap_get_rid_string (E_CAL_COMPONENT (b));
+	ret = strcmp (astr, bstr);
+	g_free (astr);
+	g_free (bstr);
+
+	return ret;
 }
 
 static void
@@ -1048,7 +1066,7 @@ collect_rids_cb (gpointer key, gpointer value, gpointer data)
 		ECalComponent *comp = l->data;
 
 		g_ptr_array_index (vd->uids, i) = g_strdup (uid);
-		g_ptr_array_index (vd->rids, i) = g_strdup (cal_backend_wcap_get_rid_string (comp));
+		g_ptr_array_index (vd->rids, i) = cal_backend_wcap_get_rid_string (comp);
 	}	
 }
 
@@ -1343,8 +1361,8 @@ write_cache_when_idle (CalBackendWCAP *wcap)
 
 	g_hash_table_foreach (priv->instances, build_calendar, icalcomp);
 	buf = icalcomponent_as_ical_string (icalcomp);
-
 	result = gnome_vfs_write (handle, buf, strlen (buf) * sizeof (char), &out);
+	g_free (buf);
 	icalcomponent_free (icalcomp);
 	gnome_vfs_close (handle);
 	if (result != GNOME_VFS_OK) {
@@ -1826,6 +1844,8 @@ go_online_with_thread (CalBackendWCAP *wcap, SunOneAccount *account)
 	sunone_connection_free_preferences (prefs);
 
 	priv->account_email_address = g_strdup (sunone_account_get_email (account));
+
+	read_cache (wcap);
 	
 	/* verify objects */
 	verify_components (wcap);
@@ -1947,7 +1967,6 @@ cal_backend_wcap_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if
 
 		return GNOME_Evolution_Calendar_Success;
 	}
-	read_cache (wcap);
 
 	thread = g_thread_create ((GThreadFunc)go_online, wcap, FALSE, &error);
 	if (!thread) {
@@ -2045,7 +2064,7 @@ cal_backend_wcap_send_objects (ECalBackendSync *backend, EDataCal *cal, const ch
 			*users = g_list_prepend (*users, tmp);
 	}
 	g_object_unref (comp);
-	*modified_calobj = g_strdup (icalcomponent_as_ical_string (top_level));
+	*modified_calobj = icalcomponent_as_ical_string (top_level);
  cleanup:	
 	icalcomponent_free (top_level);
 	
@@ -2599,7 +2618,7 @@ cal_backend_wcap_get_timezone (ECalBackendSync *backend, EDataCal *cal,
 	if (!vtzcomp)
 		return GNOME_Evolution_Calendar_OtherError;
 
-	*object = g_strdup (icalcomponent_as_ical_string (vtzcomp));
+	*object = icalcomponent_as_ical_string (vtzcomp);
 	return GNOME_Evolution_Calendar_Success;
 }
 
