@@ -47,6 +47,11 @@ struct _SunOneConnectionPrivate {
 	GHashTable *wcaps;
 };
 
+#if SUNONE_DEBUG
+gchar *verbose_debug;
+gint debug_level;
+#endif
+
 static gchar *get_acls_string (GList *acls);
 static gboolean reconnect_timeout_cb (gpointer user_data);
 
@@ -69,6 +74,13 @@ query_encode (const char *part, gboolean escape_unsafe,
 {
 	char *work, *p;
 
+	/*
+	 * return an empty string, so we can append it to
+	 * the parameter but not ignore it.
+	 */
+	if (part == NULL)
+		return g_strdup ("");
+
 	/* worst case scenario = 3 times the initial */
 	p = work = g_malloc (3 * strlen (part) + 1);
 
@@ -91,9 +103,6 @@ append_to_query (SoupURI *uri, const char *param, const char *value)
 {
 	char *tmp, *enc;
 	
-	if (!value)
-		return;
-
 	enc = query_encode (value, FALSE, " &:+");
 	tmp = g_strdup_printf ("%s&%s=%s", uri->query, param, enc);
 	g_free (enc);
@@ -186,7 +195,7 @@ append_to_query_text_list (SoupURI *uri, const char *param, GSList *text_list)
 static void
 append_to_query_datetime (SoupURI *uri, const char *param, ECalComponentDateTime *dt, gboolean zero)
 {
-	const char *string;
+	char *string;
 
 	if (!dt->value || !icaltime_is_valid_time (*dt->value)) {
 		if (zero) 
@@ -196,6 +205,7 @@ append_to_query_datetime (SoupURI *uri, const char *param, ECalComponentDateTime
 	
 	string = icaltime_as_ical_string (*dt->value);
 	append_to_query (uri, param, string);	
+	g_free (string);
 	e_cal_component_free_datetime (dt);
 }
 
@@ -339,16 +349,19 @@ append_to_query_exdate_list (SoupURI *uri, const char *param, GSList *dt_list)
 	for (l = dt_list; l != NULL; l = l->next) {
 		ECalComponentDateTime *dt = l->data;
 		char *tmp = string;
+		char *tstr;
 		
 		if (!dt->value)
 			continue;
+		tstr = icaltime_as_ical_string (*dt->value);
 		
 		if (string) {
-			string = g_strconcat (string, ";", icaltime_as_ical_string (*dt->value), NULL);
+			string = g_strconcat (string, ";", tstr, NULL);
 			g_free (tmp);
 		} else {
-			string = g_strdup (icaltime_as_ical_string (*dt->value));
+			string = g_strdup (tstr);
 		}
+		g_free (tstr);
 	}
 	if (string)
 		append_to_query (uri, param, string);
@@ -376,8 +389,7 @@ append_to_query_recur_list (SoupURI *uri, const char *param, GSList *r_list)
 		}
 		g_free (enc);
 	}
-	if (string)
-		append_to_query (uri, param, string);
+	append_to_query (uri, param, string);
 	e_cal_component_free_recur_list (r_list);
 	g_free (string);
 }
@@ -490,7 +502,7 @@ append_to_query_alarms (SoupURI *uri, SunOneConnection *connection, ECalComponen
 	ECalComponentAlarmAction action;
 	ECalComponentAlarmTrigger trigger;
 	char *atype;
-	const char *dtstring;
+	char *dtstring;
 
 	if (!e_cal_component_has_alarms (comp)) {
 		
@@ -529,7 +541,8 @@ append_to_query_alarms (SoupURI *uri, SunOneConnection *connection, ECalComponen
 		if (trigger.type == E_CAL_COMPONENT_ALARM_TRIGGER_ABSOLUTE)
 			dtstring = icaltime_as_ical_string (trigger.u.abs_time);
 		else
-			dtstring = icaldurationtype_as_ical_string (trigger.u.rel_duration);
+			dtstring =
+				g_strdup (icaldurationtype_as_ical_string (trigger.u.rel_duration));
 		
 		if (atype != NULL && dtstring != NULL) {
 			ECalComponentText text;
@@ -569,6 +582,7 @@ append_to_query_alarms (SoupURI *uri, SunOneConnection *connection, ECalComponen
 				}
 			}
 		}
+		g_free (dtstring);
 		
 		e_cal_component_alarm_free (alarm);
 	}
@@ -609,6 +623,10 @@ sunone_connection_init (SunOneConnection *object)
 {
 	SunOneConnection *connection = SUNONE_CONNECTION (object);
 	SunOneConnectionPrivate *priv;
+#if SUNONE_DEBUG
+    SoupLogger *logger;
+    SoupLoggerLogLevel level;
+#endif
 
 	connection->priv = g_new0 (SunOneConnectionPrivate, 1);
 	priv = connection->priv;
@@ -618,6 +636,15 @@ sunone_connection_init (SunOneConnection *object)
 	priv->timeout_id = -1;
 	priv->is_open = FALSE;
 	priv->wcaps = g_hash_table_new (g_str_hash, g_str_equal);
+
+#if SUNONE_DEBUG
+    if (debug_level < 4 && debug_level >= 0)
+        level = (SoupLoggerLogLevel)debug_level;
+    else
+        level = SOUP_LOGGER_LOG_BODY;
+    logger = soup_logger_new (level, -1);
+	soup_session_add_feature (priv->soup_session, SOUP_SESSION_FEATURE (logger));
+#endif
 }
 
 
@@ -2124,6 +2151,8 @@ sunone_connection_fetchcomponents_by_lastmod (SunOneConnection *connection,
 	SoupURI *fetch_uri;
 	SoupMessage *msg;
 	guint retval;
+	char *tsstr;
+	char *testr;
 
 	g_return_val_if_fail (IS_SUNONE_CONNECTION (connection), SOUP_STATUS_CANCELLED);
 	g_return_val_if_fail (calid != NULL, SUNONE_ERROR_ILLEGAL_CALID_NAME);
@@ -2132,14 +2161,23 @@ sunone_connection_fetchcomponents_by_lastmod (SunOneConnection *connection,
 
 	/* prepare the URI */
 	NEW_URI (fetch_uri);
+	if (!icaltime_is_null_time (dtstart))
+		tsstr = icaltime_as_ical_string (dtstart);
+	else
+		tsstr = g_strdup ("0");
+	if (!icaltime_is_null_time (dtend))
+		testr = icaltime_as_ical_string (dtend);
+	else
+		testr = g_strdup ("0");
 	fetch_uri->path = g_strdup ("/fetchcomponents_by_lastmod.wcap");
 	fetch_uri->query = g_strdup_printf (
 		"id=%s&calid=%s&dtstart=%s&dtend=%s&component-type=%s&relativealarm=4&fmt-out=text/calendar&compressed=1",
 		priv->wcap_session_id,
 		calid,
-		!icaltime_is_null_time (dtstart) ? icaltime_as_ical_string (dtstart) : "0",
-		!icaltime_is_null_time (dtend) ? icaltime_as_ical_string (dtend) : "0",
+		tsstr, testr,
 		type_to_string (type));
+	g_free (tsstr);
+	g_free (testr);
 
 	append_to_query_compstate (fetch_uri, "compstate", state);
 	
@@ -2181,6 +2219,8 @@ sunone_connection_fetch_deletedcomponents (SunOneConnection *connection,
 	SoupURI *fetch_uri;
 	SoupMessage *msg;
 	guint retval;
+	char *tsstr;
+	char *testr;
 
 	g_return_val_if_fail (IS_SUNONE_CONNECTION (connection), SOUP_STATUS_CANCELLED);
 	g_return_val_if_fail (calid != NULL, SUNONE_ERROR_ILLEGAL_CALID_NAME);
@@ -2189,14 +2229,23 @@ sunone_connection_fetch_deletedcomponents (SunOneConnection *connection,
 
 	/* prepare the URI */
 	NEW_URI (fetch_uri);
+	if (!icaltime_is_null_time (dtstart))
+		tsstr = icaltime_as_ical_string (dtstart);
+	else
+		tsstr = g_strdup ("0");
+	if (!icaltime_is_null_time (dtend))
+		testr = icaltime_as_ical_string (dtend);
+	else
+		testr = g_strdup ("0");
 	fetch_uri->path = g_strdup ("/fetch_deletedcomponents.wcap");
 	fetch_uri->query = g_strdup_printf (
 		"id=%s&calid=%s&dtstart=%s&dtend=%s&component-type=%s&fmt-out=text/calendar&recurring=0",
 		priv->wcap_session_id,
 		calid,
-		!icaltime_is_null_time (dtstart) ? icaltime_as_ical_string (dtstart) : "0",
-		!icaltime_is_null_time (dtend) ? icaltime_as_ical_string (dtend) : "0",
+		tsstr, testr,
 		type_to_string (type));
+	g_free (tsstr);
+	g_free (testr);
 
 	
 	/* send the request to the server */
@@ -2227,19 +2276,30 @@ sunone_connection_get_freebusy (SunOneConnection *connection,
 	SoupURI *fetch_uri;
 	SoupMessage *msg;
 	guint retval;
+	char *tsstr;
+	char *testr;
 
 	g_return_val_if_fail (IS_SUNONE_CONNECTION (connection), SOUP_STATUS_CANCELLED);
 	g_return_val_if_fail (calid != NULL, SUNONE_ERROR_ILLEGAL_CALID_NAME);
 
 	/* prepare the URI */
 	NEW_URI (fetch_uri);
+	if (!icaltime_is_null_time (dtstart))
+		tsstr = icaltime_as_ical_string (dtstart);
+	else
+		tsstr = g_strdup ("0");
+	if (!icaltime_is_null_time (dtend))
+		testr = icaltime_as_ical_string (dtend);
+	else
+		testr = g_strdup ("0");
 	fetch_uri->path = g_strdup ("/get_freebusy.wcap");
 	fetch_uri->query = g_strdup_printf (
 		"id=%s&calid=%s&dtstart=%s&dtend=%s&tzid=UTC&tzidout=UTC&busyonly=1&fmt-out=text/calendar&compressed=1",
 		priv->wcap_session_id,
 		calid,
-		!icaltime_is_null_time (dtstart) ? icaltime_as_ical_string (dtstart) : "0",
-		!icaltime_is_null_time (dtend) ? icaltime_as_ical_string (dtend) : "0");
+		tsstr, testr);
+	g_free (tsstr);
+	g_free (testr);
 
 	/* send the request to the server */
 	msg = sunone_message_new_from_uri (fetch_uri, SOUP_METHOD_GET);
@@ -2257,16 +2317,27 @@ sunone_connection_get_freebusy (SunOneConnection *connection,
 
 	/* For WCAP 3.0, maybe calid is a mail address, try again */
 	if (!IS_CNC_WCAP_2_0(connection) && !SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
+		char *tsstr;
+		char *testr;
 
 		/* prepare the URI */
 		NEW_URI (fetch_uri);
+		if (!icaltime_is_null_time (dtstart))
+			tsstr = icaltime_as_ical_string (dtstart);
+		else
+			tsstr = g_strdup ("0");
+		if (!icaltime_is_null_time (dtend))
+			testr = icaltime_as_ical_string (dtend);
+		else
+			testr = g_strdup ("0");
 		fetch_uri->path = g_strdup ("/get_freebusy.wcap");
 		fetch_uri->query = g_strdup_printf (
 			"id=%s&mail=%s&dtstart=%s&dtend=%s&tzid=UTC&tzidout=UTC&busyonly=1&fmt-out=text/calendar&compressed=1",
 			priv->wcap_session_id,
 			calid,
-			!icaltime_is_null_time (dtstart) ? icaltime_as_ical_string (dtstart) : "0",
-			!icaltime_is_null_time (dtend) ? icaltime_as_ical_string (dtend) : "0");
+			tsstr, testr);
+		g_free (tsstr);
+		g_free (testr);
 
 		/* send the request to the server */
 		msg = sunone_message_new_from_uri (fetch_uri, SOUP_METHOD_GET);
@@ -2632,6 +2703,7 @@ sunone_connection_storetodos (SunOneConnection *connection, const char *calid, i
 	icalproperty_status status;
 	const char *const_string;
 	int *num;
+	char *tstr;
 	
 	g_return_val_if_fail (IS_SUNONE_CONNECTION (connection), SOUP_STATUS_CANCELLED);
 	g_return_val_if_fail (calid != NULL, SUNONE_ERROR_ILLEGAL_CALID_NAME);
@@ -2671,7 +2743,12 @@ sunone_connection_storetodos (SunOneConnection *connection, const char *calid, i
 
 	/* Completed */
 	e_cal_component_get_completed (comp, &t);
-	append_to_query (fetch_uri, "completed", t ? icaltime_as_ical_string (*t) : "0");
+	if (t)
+		tstr = icaltime_as_ical_string (*t);
+	else
+		tstr = g_strdup ("0");
+	append_to_query (fetch_uri, "completed", tstr);
+	g_free (tstr);
 	
 	/* Description */
 	e_cal_component_get_description_list (comp, &slist);
